<?php

/*
    -------------------------------------------------------------------
    moziloCMS-Plugin für Links auf Anker
    -------------------------------------------------------------------
    
    Details und Dokumentation: http://software.azett.com
    
*/

class ankerlink extends Plugin {

    function getContent($value) {
        // Kein Parameter übergeben?
        if ($value == "") {
            return $this->buildErrorMsg("Kein Parameter übergeben.");
        }
        
        // Erstmal den Linktext raussuchen
        $params = explode("||", $value);
        $paramcount = count($params);
        if ($paramcount > 1) {
            $linktext=$params[0];
            $subvalue = $params[1];
        }
        else {
            $linktext = "";
            $subvalue = $value;
        }
        // >, < oder ? als Text angegeben? -> Sonderfunktion "Ankerlinks für spezifische Elemente" (erste/letzte/zufällige)
        if($linktext == "&lt;") {
            return $this->getAnchorLinksForSpecificElements($subvalue, 0);
        }
        elseif($linktext == "&gt;") {
            return $this->getAnchorLinksForSpecificElements($subvalue, 1);
        }
        elseif($linktext == "?") {
            return $this->getAnchorLinksForSpecificElements($subvalue, 2);
        }
        // sonst normaler Ankerlink
        else {
            return $this->getAnchorLink($linktext, $subvalue);
        }
    } // function getContent



    // Hauptfunktion des Plugins: Link auf einen Anker erstellen
    function getAnchorLink($linktext, $subvalue) {
        global $CAT_REQUEST_URL;
        global $specialchars;

        // die eigentlichen Anker-Parameter
		$subparams = explode("|", $subvalue);	
        // Kein Sub-Parameter übergeben?
        if ($subvalue == "") {
            return $this->buildErrorMsg("Anzeigetext, aber kein Ziel übergeben.");
        }
        $subparamcount = count($subparams);

        $catparam    = "";
        $pageparam   = "";
        // Der Anker ist immer der letzte Parameter
        $anchorparam = $subparams[$subparamcount-1];
        $anchordesc = "";
        // beginnt der Anker-Parameter mit einer Raute?
        if (substr($anchorparam, 0, 1) == "#") {
            // dann wird das auch der anzuspringende Anker
            $anchor = $anchorparam;
        }
        else {
            // sonst ists die anzuspringende Überschrift; für die den Ankernamen bilden
            $anchor = "#".$this->calculateId($anchorparam);
        }
        
        // Kein Linktext angegeben? Dann Ankernamen nehmen
        if ($linktext == "") {
            $linktext = $anchorparam;
        }
        
        // Ausgabe!
        // Zuviele Parameter!
        if (count($subparams) > 3) {
            return $this->buildErrorMsg("Maximal drei Parameter möglich: Kategorie, Seite, Anker.");
        }
        if (count($subparams) > 2) {
            $catparam = $specialchars->replaceSpecialChars($specialchars->getHtmlEntityDecode($subparams[$subparamcount-3]), false);
            $pageparam = $specialchars->replaceSpecialChars($specialchars->getHtmlEntityDecode($subparams[$subparamcount-2]), false);
            return "<a href=\"index.php?cat=$catparam&amp;page=$pageparam$anchor\" class=\"plugin_ankerlink\" title=\"Springe zu Absatz &quot;".$anchorparam."&quot; auf Seite &quot;".$subparams[$subparamcount-2]."&quot; in Kategorie &quot;".$subparams[$subparamcount-3]."&quot;\">$linktext</a>";
        }
        if (count($subparams) > 1) {
            $pageparam = $specialchars->replaceSpecialChars($specialchars->getHtmlEntityDecode($subparams[$subparamcount-2]), false);
            return "<a href=\"index.php?cat=$CAT_REQUEST_URL&amp;page=$pageparam$anchor\" class=\"plugin_ankerlink\" title=\"Springe zu Absatz &quot;".$anchorparam."&quot; auf Seite &quot;".$subparams[$subparamcount-2]."&quot;\">$linktext</a>";
        }
        return "<a href=\"$anchor\" class=\"plugin_ankerlink\" title=\"Springe zu Absatz &quot;".$anchorparam."&quot;\">$linktext</a>";
    } // function getAnchorLink



    // Ankerlinks für spezifische Elemente
    // $mode: 
    //  - 0 = erste X Elemente
    //  - 1 = letzte X Elemente
    //  - 2 = X zufällige Elemente
    function getAnchorLinksForSpecificElements($value, $mode) {
        global $specialchars;
        global $CatPage;

        // Kein Parameter übergeben?
        if ($value == "" or $value === false) {
            return $this->buildErrorMsg("Kein Parameter übergeben.");
        }
        // Parameter auslesen
		$params = explode("|", $value);	
        $paramcount = count($params);
        
        // Zuviele Parameter?
        if (count($params) != 3) {
            return $this->buildErrorMsg("Ungültige Anzahl Parameter! Drei werden benötigt: Kategorie:Seite, Syntaxelement, Anzahl.");
        }
        
        // Keine Ganzzahl >= 0 angegeben?
        if(!preg_match("/^[0-9]+$/i", $params[2])) {
            return $this->buildErrorMsg("Der dritte Parameter muss eine Ganzzahl >= 0 sein.");
        }
        
        // Kategorie und Seite
        list($catparam, $pageparam) = $CatPage->split_CatPage_fromSyntax($params[0]);
        
        // Kategorie und Inhaltsseite vorhanden?
        if (!$CatPage->exists_CatPage($catparam,$pageparam)) {
            return $this->buildErrorMsg("Angegebene Kategorie oder Inhaltsseite nicht gefunden.");
        }

        // Seiteninhalt holen!
        $pagecontent = $CatPage->get_PageContent($catparam, $pageparam);
        
        // Alle Vorkommen des gesuchten Syntaxelements
        preg_match_all("/\[".$params[1]."(\=[^\[\]\{\}]*)?\|([^\[\]\{\}]*)\]/u", $pagecontent, $matches);
        // die Inhalte der gefundenen Syntaxelemente extrahieren
        $contents = $matches[2];

        // erste/letzte/zufällige X Elemente zusammensuchen
        switch ($mode) {
            case 0: // erste X Elemente
                $contentsToKeep = array_slice($contents, 0, $params[2]); 
                // Reihenfolge umkehren
                if ($this->settings->get("specificelements_reverseorder") == "true")  {
                    $contentsToKeep = array_reverse($contentsToKeep);
                }
                break;
            case 1: // letzte X Elemente
                $contentsToKeep = array_slice($contents, -$params[2], $params[2]); 
                // Reihenfolge umkehren
                if ($this->settings->get("specificelements_reverseorder") == "true")  {
                    $contentsToKeep = array_reverse($contentsToKeep);
                }
                break;
            case 2: // X zufällige Elemente
                shuffle($contents);
                $contentsToKeep = array_slice($contents, 0, $params[2]); 
                break;
        }

        // HTML-Ausgabe zusammenbauen
        $returnstring = $this->settings->get("specificelements_overall_prefix");
        for ($i=0; $i < count($contentsToKeep); $i++) {
                $returnstring .= $this->buildAnchorLinkForOneSpecificElement($catparam, $pageparam, $contentsToKeep[$i]);
        }
        return $returnstring.$this->settings->get("specificelements_overall_postfix");
    } // function getAnchorLinksForSpecificElements



    // Hilfsfunktion: Baut den Ankerlink eines einzelnen Elements zusammen
    function buildAnchorLinkForOneSpecificElement($catparam, $pageparam, $content) {
            $anchorid = "#".$this->calculateId($content);
            return $this->settings->get("specificelements_each_prefix")
                ."<a href=\"index.php?cat=$catparam&amp;page=$pageparam$anchorid\" class=\"plugin_ankerlink\""
            /* 
            TODO Arvid: Kategorie- und Seitennamen fürs title-Attribut holen!
            ." /*title=\"Springe zu Absatz &quot;".$content."&quot; auf Seite &quot;".$subparams[$subparamcount-2]."&quot; in Kategorie &quot;".$subparams[$subparamcount-3]."&quot;\""
            */
                .">".$content."</a>"
                .$this->settings->get("specificelements_each_postfix");
    }



    function getConfig() {
        $config = array();
        // Schalter "Erste/Letzte Elemente"
        $config['specificelements_overall_prefix']  = array(
            "type" => "text",
            "description" => "\"Ankerlinks für spezifische Elemente\" - Präfix für gesamte Ausgabe"
        );
        $config['specificelements_each_prefix']  = array(
            "type" => "text",
            "description" => "\"Ankerlinks für spezifische Elemente\" - Präfix für jeden Ankerlink"
        );
        $config['specificelements_each_postfix']  = array(
            "type" => "text",
            "description" => "\"Ankerlinks für spezifische Elemente\" - Postfix für jeden Ankerlink"
        );
        $config['specificelements_overall_postfix']  = array(
            "type" => "text",
            "description" => "\"Ankerlinks für spezifische Elemente\" - Postfix für gesamte Ausgabe"
        );
        $config['specificelements_reverseorder'] = array(
            "type" => "checkbox",
            "description" => "\"Ankerlinks für spezifische Elemente\" - Ausgabe-Reihenfolge umkehren?"
        );
        return $config;
    } // function getConfig    
    
    
    function getInfo() {
        $descriptiontext = "Erstellt Links auf HTML-Anker. <br />
            Ideal zur Ergänzung des Plugins \"Individuelle Anker\"!<br />
            <br />"
            .$this->boldspan("Nutzung:")."<br />"
            .$this->codespan("{ankerlink|#a1}")." - Anker in aktueller Seite<br />"
            .$this->codespan("{ankerlink|Seite|#a1}")." - Anker in anderer Seite<br />"
            .$this->codespan("{ankerlink|Kategorie|Seite|#a1}")." - Anker in anderer Seite in anderer Kategorie<br />
            <br />"
            .$this->boldspan("Verbindung mit dem Plugin \"Individuelle Anker\":")."<br />
            Alternativ zu den exakten Anker-IDs, auf die verlinkt werden soll, können (für alle beschriebenen Möglichkeiten) auch Ankernamen des Plugins \"Individuelle Anker\" genutzt werden. Beispiel:<br />"
            .$this->codespan("{ankerlink|Seite|Mein Anker!}")." - Link auf den individuellen Anker mit dem Namen \"Mein Anker!\" (ID \"a_Mein_Anker_\") in der Inhaltsseite \"Seite\"<br />
            <br />"
            .$this->boldspan("Beliebiger Anzeigetext:")."<br />
            Für alle bisher beschriebenen Möglichkeiten kann auch beliebiger Anzeigetext übergeben werden. Der Anzeigetext wird mit doppelter Pipe (\"||\") getrennt VOR allen anderen Parametern notiert. Beispiel:<br />"
            .$this->codespan("{ankerlink|TEXT||Willkommen|Impressum|AGBs}")." - Link auf Anker \"AGBs\" in der Inhaltsseite \"Impressum\" in der Kategorie \"Willkommen\"<br />
            <br />"
            .$this->boldspan("Ankerlinks für spezifische Elemente:")."<br />
            Mit dieser zusätzlichen Funktion kann man sich Ankerlinks auf...<br />
            • die ersten X Elemente<br />
            • die letzten X Elemente<br />
            • X zufällige Elemente<br />
            ...einer Inhaltsseite ausgeben lassen. <br />
            <br />
            Hat man bspw. eine Inhaltsseite mit mehreren Abschnitten, die immer mit einer großen Überschrift [ueber1|...] eingeleitet werden, so kann man sich z.B. Ankerlinks auf die drei ersten großen Überschriften ausgeben lassen:<br />"
            .$this->codespan("{ankerlink|&#60;||Kategorie:Inhaltsseite|ueber1|3}")."<br />
            Der erste Parameter des Plugins bestimmt den Modus: ".$this->codespan("&#60;")." steht für die ersten X Elemente der Inhaltsseite, ".$this->codespan("&#62;")."für die letzten X und ".$this->codespan("?")."für X zufällige.<br />
            <br />
            Die Prä- und Postfixe für die einzelnen Links und für die Gesamtausgabe lassen sich einzeln konfigurieren, um z.B. eine syntaktisch korrekte HTML-Liste zu erstellen (Standard im Auslieferungszustand des Plugins).<br />
            ";
            
        $info = array(
            // Plugin-Name + Version
            "<b>Anker-Link</b> 1.1 (Revision: 156)",
            // moziloCMS-Version
            "1.12",
            // Kurzbeschreibung nur <span> und <br /> sind erlaubt
            $descriptiontext,
            // Name des Autors
            "Arvid Zimmermann",
            // Download-URL
            "http://software.azett.com",
            // Platzhalter für die Selectbox in der Editieransicht 
            // - ist das Array leer, erscheint das Plugin nicht in der Selectbox
            array(
                '{ankerlink|}' => 'Anker-Link'
            )
        );
        return $info;
        
    } // function getInfo
    
    // Anker-ID so erstellen, daß sie HTML-valide ist 
    // - muß mit Buchstaben beginnen
    // - erlaubte Zeichen: a-z A-Z 0-9 - _ : .
    // - nicht erlaubte Zeichen werden durch Unterstrich ersetzt
    function calculateId($content) {
        $regex = "a-zA-Z0-9\-\_\:\.";
        return "a_".preg_replace("/[^".$regex."\.]/i", "_", html_entity_decode($content));
    }

    // Zusatzfunktion: Gibt Fehlermeldung aus
	function buildErrorMsg($msg) {
		return '<span class="deadlink">Fehler im Plugin ankerlink: '.$msg.'</span>';
	}
    
    // Text in Code-Span packen
    function codespan($content) {
        return "<span style=\"font-family:monospace;margin:3px 3px; background-color:#eee;border:1px dotted #aaa;\">".$content."</span>";
    }

    // Text in Bold-Span packen
    function boldspan($content) {
        return "<span style=\"font-weight:bold;\">".$content."</span>";
    }
} // class ankerlink

?>